package convert

import (
	"fmt"
	"git.chillcy.com/golang/chillcy/pkg/currency"
	"git.chillcy.com/golang/chillcy/pkg/gender"
	math2 "git.chillcy.com/golang/chillcy/pkg/math"
	"git.chillcy.com/golang/chillcy/pkg/platform"
	"git.chillcy.com/golang/chillcy/pkg/sdk/pc3"
	"git.chillcy.com/golang/chillcy/pkg/slice"
	"git.chillcy.com/golang/chillcy/project/pc4/pkg/rpc/pb/pc4"
	"git.chillcy.com/golang/colorway/internal/pkg/database/mongo"
	siteGroupMarginConfigModel "git.chillcy.com/golang/colorway/internal/pkg/database/mongo/model/SiteGroupMarginConfig"
	productModel "git.chillcy.com/golang/colorway/internal/pkg/database/mongo/model/product"
	siteComboMarginConfigModel "git.chillcy.com/golang/colorway/internal/pkg/database/mongo/model/site_combo_margin_config"
	siteFloatMarginConfigModel "git.chillcy.com/golang/colorway/internal/pkg/database/mongo/model/site_float_margin_config"
	sitePlatformPriceConfigModel "git.chillcy.com/golang/colorway/internal/pkg/database/mongo/model/site_platform_price_config"
	subjectModel "git.chillcy.com/golang/colorway/internal/pkg/database/mongo/model/subject"
	"git.chillcy.com/golang/colorway/internal/rpc/internal/svc/cache"
	"git.chillcy.com/golang/colorway/internal/rpc/internal/svc/priceRouteAdapter"
	"git.chillcy.com/golang/colorway/internal/rpc/internal/svc/rate"
	"git.chillcy.com/golang/colorway/pkg/types"
	"math"
	"regexp"
	"slices"
	"strconv"
	"time"
)

type PlatformStoreSku struct {
	mongoManager   *mongo.Manager
	rate           rate.Rate
	brandRankCache cache.BrandRank
}

func NewPlatformStoreSku(mongoManager *mongo.Manager, rate rate.Rate, brandRankCache cache.BrandRank) *PlatformStoreSku {
	return &PlatformStoreSku{mongoManager: mongoManager, rate: rate, brandRankCache: brandRankCache}
}

// Convert 转换单个 SKU 的所有平台-店铺-路线价格
func (receiver *PlatformStoreSku) Convert(product *pc4.ProductModel, sku *pc4.SkuModel, oldProduct *productModel.Product) (productModel.ProductColorSkuPlatformStoreSkus, error) {
	var platformStoreSkus productModel.ProductColorSkuPlatformStoreSkus

	sitePlatformPriceConfigs := receiver.mongoManager.SitePlatformPriceConfigModel.GetCacheMany(func(document sitePlatformPriceConfigModel.SitePlatformPriceConfig) bool {
		return document.Site == sku.GetSite()
	})
	categoryIds := product.GetCategoryIds()
	for _, sitePlatformPriceConfig := range sitePlatformPriceConfigs {
		var pc3SkuModel pc3.ModelSku
		platformPriceSku, err := receiver.calculatePlatformPrice(product, sku, oldProduct, pc3SkuModel, categoryIds, sitePlatformPriceConfig)
		if err != nil {
			return nil, err
		}
		platformStoreSkus = append(platformStoreSkus, platformPriceSku)
	}

	return platformStoreSkus, nil
}

func (receiver *PlatformStoreSku) calculatePlatformPrice(product *pc4.ProductModel, sku *pc4.SkuModel, oldProduct *productModel.Product, pc3SkuModel pc3.ModelSku, categories []string, priceConfig sitePlatformPriceConfigModel.SitePlatformPriceConfig) (productModel.ProductColorSkuPlatformStoreSku, error) {
	var psSku productModel.ProductColorSkuPlatformStoreSku
	if !priceConfig.IsValid() {
		return psSku, fmt.Errorf("路线价格配置不合法")
	}
	psSku, err := receiver.calculateRoutePrice(product, sku, oldProduct, pc3SkuModel, categories, priceConfig.RoutePriceConfig, priceConfig.Seller)
	if err != nil {
		return psSku, err
	}
	return psSku, nil
}

func (receiver *PlatformStoreSku) calculateRoutePrice(product *pc4.ProductModel, sku *pc4.SkuModel, oldProduct *productModel.Product, pc3SkuModel pc3.ModelSku, categories []string, routeConfig sitePlatformPriceConfigModel.RoutePriceConfig, seller types.Seller) (productModel.ProductColorSkuPlatformStoreSku, error) {
	var platformStoreSku productModel.ProductColorSkuPlatformStoreSku

	adapter, err := priceRouteAdapter.SelectAdapter(routeConfig.Route)
	if err != nil {
		return platformStoreSku, err
	}

	currencyRate, err := receiver.rate.GetRate(seller.Store, pc3SkuModel.PriceDetail.Currency, routeConfig.Currency.String())
	if err != nil {
		return platformStoreSku, err
	}

	subjects := receiver.mongoManager.SubjectModel.GetCacheMany(func(document subjectModel.Subject) bool {
		return slices.Contains(product.GetSubjects(), document.Id.Hex())
	})
	var subjectRate, subjectId = float64(1), ""
	for _, subject := range subjects {
		if subject.PriceRate < subjectRate {
			subjectRate = subject.PriceRate
			subjectId = subject.Id.Hex()
		}
	}

	// 固定毛利
	var fixMargin float64
	if oldProduct != nil {
		if platformStoreSku, ok := oldProduct.FindPlatformStoreSku(seller.Platform, seller.Store, routeConfig.Route, sku.GetId()); ok {
			if platformStoreSku.FixMargin != nil {
				fixMargin = *platformStoreSku.FixMargin
			}
		}
	}
	margin := 1.0
	if fixMargin != 0 {
		margin = 1 + (fixMargin / 100)
	} else {
		margin, err = receiver.calculateMargin(product, sku, routeConfig, pc3SkuModel, seller)
		if err != nil {
			return platformStoreSku, err
		}
	}
	margin = math2.Round(math.Max(1.0, margin), 3) // 保底1.0
	freight, err := receiver.calculateFreight(pc3SkuModel, categories, routeConfig.FreightConfig, seller.Store)
	if err != nil {
		return platformStoreSku, err
	}
	priceDetail := priceRouteAdapter.RoutePriceDetail{
		Route:          routeConfig.Route,
		Currency:       string(routeConfig.Currency),
		OriginCurrency: pc3SkuModel.PriceDetail.Currency,
		Rate:           currencyRate,
		Margin:         margin,
		SubjectRate:    subjectRate,
		Subject:        subjectId,
		DiscountRate:   pc3SkuModel.PriceDetail.Discount,
		TotalDiscount:  pc3SkuModel.PriceDetail.Discount * subjectRate,
		TaxRate:        1 + routeConfig.TaxRate,
		Freight:        freight,
		Express:        receiver.calculateExpress(categories, routeConfig.ExpressConfig),
	}

	// 计算最终价格
	priceDetail.Price, err = adapter.GetPrice(pc3SkuModel.PriceDetail.Now, priceDetail, platform.Platform(seller.Platform))
	if err != nil {
		return platformStoreSku, err
	}

	cnyRate, err := receiver.rate.GetRate(seller.Store, priceDetail.Currency, currency.CNY.String())
	if err != nil {
		return platformStoreSku, err
	}

	platformStoreSku = productModel.ProductColorSkuPlatformStoreSku{
		Platform:       seller.Platform,
		Store:          seller.Store,
		Route:          priceDetail.Route,
		Currency:       priceDetail.Currency,
		OriginCurrency: priceDetail.OriginCurrency,
		Price:          priceDetail.Price,
		//FixPrice:       priceDetail,
		//FixMargin:      nil,
		Rate:          priceDetail.Rate,
		CnyRate:       cnyRate,
		Margin:        priceDetail.Margin,
		SubjectRate:   priceDetail.SubjectRate,
		Subject:       priceDetail.Subject,
		DiscountRate:  priceDetail.DiscountRate,
		TotalDiscount: priceDetail.TotalDiscount,
		TaxRate:       priceDetail.TaxRate,
		Freight:       priceDetail.Freight,
		Express:       priceDetail.Express,
		//ChangeRate:     nil,
		//ChangeAt:       nil,
		//Memos:          nil,
	}

	// todo
	//if oldSkuPriceRecord != nil {
	//	now := time.Now()
	//	change := math2.Round((priceDetail.Price-oldSkuPriceRecord.Price)/oldSkuPriceRecord.Price, 4)
	//	record.Id = oldSkuPriceRecord.Id
	//	record.FixedQty = oldSkuPriceRecord.FixedQty
	//	if math.Abs(change) > 0.0001 {
	//		record.ChangeRate = &change
	//		record.ChangeAt = &now
	//	}
	//}
	return platformStoreSku, nil
}

func (receiver *PlatformStoreSku) calculateExpress(categoryIds []string, config sitePlatformPriceConfigModel.ExpressConfig) float64 {
	if config.ExpressStyle == sitePlatformPriceConfigModel.ExpressStyleDefault {
		return config.Express
	} else if config.ExpressStyle == sitePlatformPriceConfigModel.ExpressStyleCategory {
		opt := 50.0 // 操作费
		fee := 0.0 + opt

		// 没有分类的定为 30
		if len(categoryIds) <= 0 {
			return fee + 30
		}

		if receiver.isSuitcases(categoryIds) {
			return fee + 299 // 箱包固定 299
		}

		if receiver.isUmbrellasBoots(categoryIds) {
			fee = 25 // 雨伞多收 25
		}

		// 按重量计算
		weight := receiver.getWeightByCategory(categoryIds)

		if weight >= 4 {
			fee += 70
		} else if weight == 3 {
			fee += 55
		} else if weight == 2 {
			fee += 42
		} else {
			fee += 30
		}

		return fee
	}
	return 0
}

// 特别细分品类预估重量，优先级比上面的一级要高
var specialWeightMapping = map[string]int{
	"boots":               3, // 靴子
	"sneaker":             3, // 运动鞋
	"coat":                2, // 外套
	"jumpsuits-playsuits": 2, // 连身裤
	"suit":                2, // 西装
	"hat":                 2, // 帽子
	"denim":               2, //牛仔服装
}

// 一级分类基本预估重量
var basicWeightMapping = map[string]int{
	"accessory": 1, // 配饰
	"bag":       2, // 包袋
	"clothing":  1, // 服装
	"gear":      1, // 装备
	"shoes":     2, // 鞋类
	"home":      1, // 居家日用
}

func (receiver *PlatformStoreSku) getWeightByCategory(categoryIds []string) int {
	for _, categoryId := range categoryIds {
		if weight, ok := specialWeightMapping[categoryId]; ok {
			return weight
		}
	}

	for _, categoryId := range categoryIds {
		if weight, ok := basicWeightMapping[categoryId]; ok {
			return weight
		}
	}

	return 1
}

// 长柄雨伞
var umbrellas = []string{
	"umbrella", // 雨伞
}

func (receiver *PlatformStoreSku) isUmbrellasBoots(categoryIds []string) bool {
	for _, categoryId := range categoryIds {
		if slice.Contains(umbrellas, categoryId) {
			return true
		}
	}

	return false
}

// 行李箱
var suitCases = []string{
	"suitcase", //  拉杆箱
}

func (receiver *PlatformStoreSku) isSuitcases(categoryIds []string) bool {
	for _, categoryId := range categoryIds {
		if slice.Contains(suitCases, categoryId) {
			return true
		}
	}

	return false
}

func (receiver *PlatformStoreSku) calculateFreight(pc3Sku pc3.ModelSku, categoryIds []string, config sitePlatformPriceConfigModel.FreightConfig, store types.Store) (float64, error) {
	switch config.FreightStyle {
	case sitePlatformPriceConfigModel.FreightStyleDefault:
		return config.Freight, nil
	case sitePlatformPriceConfigModel.FreightStyleCategory:
		// 配饰 或者（服装下不属于 外套、礼服、针织衫）的分类
		if slice.Contains(categoryIds, "accessory") ||
			(slice.Contains(categoryIds, "clothing") &&
				!slice.Contains(categoryIds, "outwear") &&
				!slice.Contains(categoryIds, "dress") &&
				!slice.Contains(categoryIds, "knitwear")) {
			return 80, nil
		}
		// 其他分类就用250
		return 100, nil
	case sitePlatformPriceConfigModel.FreightStylePrice:
		if pc3Sku.PriceDetail.Price < 1500 {
			return 30, nil
		} else if pc3Sku.PriceDetail.Price < 2500 {
			return 50, nil
		} else if pc3Sku.PriceDetail.Price < 3500 {
			return 80, nil
		} else {
			return 100, nil
		}
	case sitePlatformPriceConfigModel.FreightStyleMinPrice:
		// 统一转成人民币价格后再计算
		currencyRate, err := receiver.rate.GetRate(store, config.OverFreightCurrency.String(), currency.CNY.String())
		if err != nil {
			return 0, err
		}
		limitPrice := float64(config.OverFreight) * currencyRate
		goodCnyPrice := pc3Sku.PriceDetail.Price * pc3Sku.PriceDetail.SubjectRate
		if goodCnyPrice >= limitPrice {
			return 0, nil
		}
		return config.Freight, nil
	}
	return 0, nil
}

func StringSliceToGenders(gs []string) gender.Genders {
	var genders gender.Genders
	for _, g := range gs {
		genders = append(genders, gender.Gender(g))
	}
	return genders
}

func (receiver *PlatformStoreSku) calculateMargin(product *pc4.ProductModel, sku *pc4.SkuModel, routeConfig sitePlatformPriceConfigModel.RoutePriceConfig, pc3Sku pc3.ModelSku, seller types.Seller) (float64, error) {

	floatMargin := 1.0
	if routeConfig.Route == types.RouteComplex {

		// 尝试获取品牌性别品类组合加价配置
		comboMarginConfig, ok := receiver.mongoManager.SiteComboMarginConfigModel.GetCacheOne(func(document siteComboMarginConfigModel.SiteComboMarginConfig) bool {
			return document.Seller.Platform == seller.Platform &&
				document.Seller.Store == seller.Store && !document.Enable &&
				document.SiteId == sku.GetSite() &&
				slices.Contains(document.Brands, product.GetBrandId()) &&
				document.IsGenderMatch(StringSliceToGenders(product.GetGenders())) &&
				slices.Contains(document.Categories, product.GetCategoriesV2().GetCategoryId())
		})
		if ok {
			return 1 + (comboMarginConfig.Margin / 100), nil
		}

		// 尝试解析浮动加价率配置
		siteFloatMarginConfig, ok := receiver.mongoManager.SiteFloatMarginConfigModel.GetCacheOne(func(document siteFloatMarginConfigModel.SiteFloatMarginConfig) bool {
			return document.Seller.Platform == seller.Platform && document.Seller.Store == seller.Store
		})
		if ok {
			rules, err := siteFloatMarginConfig.Explain()
			if err != nil {
				return 0, err
			}
			for _, rule := range rules {
				if ok, v := receiver.IsFloatMarginConfigMatch(rule, product); ok {
					if v != "" {
						intValue, _ := strconv.Atoi(v)
						flotV := 1 + (float64(intValue) / 100)
						floatMargin *= flotV
						//log.Println("命中浮动加价率配置:", rule.Type, flotV)
					}
				}
			}
		}

		// 尝试获取站点分组加价配置
		siteGroupMarginConfig, ok := receiver.mongoManager.SiteGroupMarginConfigModel.GetCacheOne(func(document siteGroupMarginConfigModel.SiteGroupMarginConfig) bool {
			return document.Seller.Platform == seller.Platform && document.Seller.Store == seller.Store &&
				document.IsMatch(sku.GetSite())
		})
		if ok {
			return (1 + (siteGroupMarginConfig.Margin / 100)) * floatMargin, nil
		}
	}
	// 保底站点加价率 * 浮动
	marginConfig := routeConfig.MarginConfig
	switch marginConfig.MarginStyle {
	case sitePlatformPriceConfigModel.MarginStyleFixed:
		return (1 + marginConfig.Margin) * floatMargin, nil
	case sitePlatformPriceConfigModel.MarginStyleLadder:
		if marginConfig.IsValidLadders() {
			marginConfig.FormatLadders()
			for _, ladder := range marginConfig.Ladders {
				if ladder.IsMatch(pc3Sku.PriceDetail.Price) {
					return (1 + ladder.Margin) * floatMargin, nil
				}
			}
		}
		return 0, fmt.Errorf("阶梯加价未能找到合适的价格区间匹配加价率")
	default:
		return 1, nil
	}
}

func (receiver *PlatformStoreSku) IsFloatMarginConfigMatch(cr siteFloatMarginConfigModel.ConditionRule, p *pc4.ProductModel) (bool, string) {
	if cr.Type == "season" && p.Season != "" {
		if p.Season == "CO" {
			return true, cr.Get(siteFloatMarginConfigModel.ConditionKeySeasonCO)
		}
		reg := "(FW|SS)(\\d{2})"
		if ok, _ := regexp.MatchString(reg, p.Season); ok {
			nowMonth := time.Now().Month()
			nowYear := time.Now().Year() - 2000
			prefix := "SS"
			if nowMonth > 6 {
				prefix = "FW"
			}
			yearInSeason, _ := strconv.Atoi(p.Season[2:])
			prefixInSeason := p.Season[:2]
			if yearInSeason > nowYear {
				return true, cr.Get(siteFloatMarginConfigModel.ConditionKeySeasonNow)
			}
			if yearInSeason == nowYear && (prefixInSeason == prefix || prefixInSeason == "FW") {
				return true, cr.Get(siteFloatMarginConfigModel.ConditionKeySeasonNow)
			}
			return true, cr.Get(siteFloatMarginConfigModel.ConditionKeySeasonOld)
		}
	}
	if cr.Type == "brand" {
		if p.GetBrandId() == "" {
			return true, cr.Get(siteFloatMarginConfigModel.ConditionKeyBrandTail)
		} else {
			topBrands, middleBrands := receiver.brandRankCache.GetBrandRankIds()
			if slice.Contains(topBrands, p.GetBrandId()) {
				return true, cr.Get(siteFloatMarginConfigModel.ConditionKeyBrandTop)
			}
			if slice.Contains(middleBrands, p.GetBrandId()) {
				return true, cr.Get(siteFloatMarginConfigModel.ConditionKeyBrandMiddle)
			}
			return true, cr.Get(siteFloatMarginConfigModel.ConditionKeyBrandTail)
		}
	}
	if cr.Type == "category" && p.GetCategoriesV2().GetCategoryId() != "" {
		if slice.Contains([]string{"clothing", "bag", "shoes", "accessory"}, p.GetCategoriesV2().GetCategoryId()) {
			return true, cr.Get("category_" + p.GetCategoriesV2().GetCategoryId())
		}
	}
	return false, ""
}
